@@ -18,13 +18,14 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: RTIG.cc,v 3.39 2008/04/08 14:18:17 rousse Exp $
+// $Id: RTIG.cc,v 3.49 2008/07/10 20:20:05 approx Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "RTIG.hh"
 
 #include "PrettyDebug.hh"
+#include "NM_Classes.hh"
 
 #ifdef _WIN32
 #include <signal.h>
@@ -32,8 +33,9 @@
 #include <unistd.h>
 #include <iostream>
 #endif
-#include <errno.h>
+#include <cerrno>
 #include <csignal>
+#include <memory>
 
 using std::cout ;
 using std::endl ;
@@ -42,14 +44,16 @@ using std::cerr ;
 namespace certi {
 namespace rtig {
 
-static pdCDebug D("RTIG", __FILE__);
+static PrettyDebug D("RTIG", __FILE__);
 static PrettyDebug G("GENDOC",__FILE__);
 
+bool RTIG::terminate = false;
+
 // ----------------------------------------------------------------------------
 // Constructor
 // Note (JYR) : udpPort modified AFTER used in socketServer(.) call, strange...
 RTIG::RTIG()
-    : terminate(false), federationHandles(1),
+    :  federationHandles(1),
       socketServer(&tcpSocketServer, &udpSocketServer, udpPort),
       auditServer(RTIG_AUDIT_FILENAME),
       federations(socketServer, auditServer)
@@ -84,15 +88,12 @@ RTIG::~RTIG()
 Socket*
 RTIG::chooseProcessingMethod(Socket *link, NetworkMessage *msg)
 {
-    G.Out(pdGendoc,"enter RTIG::chooseProcessingMethod type %d",msg->type);
+    G.Out(pdGendoc,"enter RTIG::chooseProcessingMethod type (%s)",msg->getName().c_str());
     // This may throw a security error.
-    if ( msg->type != NetworkMessage::DESTROY_FEDERATION_EXECUTION)
-       socketServer.checkMessage(link->returnSocket(), msg);
-
-	//D.Mes(pdMessage, 'N', msg->type);
-	msg->trace("RTIG::chooseProcessingMethod ");
+    if ( msg->getType() != NetworkMessage::DESTROY_FEDERATION_EXECUTION)
+       socketServer.checkMessage(link->returnSocket(), msg);	
 	
-    switch(msg->type) {
+    switch(msg->getType()) {
       case NetworkMessage::MESSAGE_NULL:
         D.Out(pdDebug, "Message Null.");
         auditServer.setLevel(0);
@@ -120,20 +121,20 @@ RTIG::chooseProcessingMethod(Socket *link, NetworkMessage *msg)
         break ;
 
       case NetworkMessage::CREATE_FEDERATION_EXECUTION:
-        D.Out(pdTrace, "Create federation \"%s\".", msg->federationName);
+        D.Out(pdTrace, "Create federation \"%s\".", msg->federationName.c_str());
         auditServer.setLevel(9);
         processCreateFederation(link, msg);
         break ;
 
       case NetworkMessage::DESTROY_FEDERATION_EXECUTION:
-        D.Out(pdTrace, "Destroy federation \"%s\".", msg->federationName);
+        D.Out(pdTrace, "Destroy federation \"%s\".", msg->federationName.c_str());
         auditServer.setLevel(9);
         processDestroyFederation(link, msg);
         break ;
 
       case NetworkMessage::JOIN_FEDERATION_EXECUTION:
         D.Out(pdTrace, "federate \"%s\" joins federation \"%s\".",
-              msg->federateName, msg->federationName);
+              msg->federateName.c_str(), msg->federationName.c_str());
         auditServer.setLevel(9);
         processJoinFederation(link, msg);
         break ;
@@ -149,8 +150,8 @@ RTIG::chooseProcessingMethod(Socket *link, NetworkMessage *msg)
         D.Out(pdTrace,
               "Federation %u: registerFedSyncPoint from federate %u.",
               msg->federation, msg->federate);
-        auditServer.setLevel(8);
-        processRegisterSynchronization(link, msg);
+        auditServer.setLevel(8);        
+        processRegisterSynchronization(link, static_cast<NM_Register_Federation_Synchronization_Point*>(msg));
         break ;
 
       case NetworkMessage::SYNCHRONIZATION_POINT_ACHIEVED:
@@ -204,15 +205,16 @@ RTIG::chooseProcessingMethod(Socket *link, NetworkMessage *msg)
 
       case NetworkMessage::SET_TIME_REGULATING:
         D.Out(pdTrace, "SetTimeRegulating du federe %u(date=%f).",
-              msg->federate, msg->date);
+              msg->federate, msg->getDate());
         auditServer.setLevel(8);
-        processSetTimeRegulating(msg);
+        processSetTimeRegulating(link, static_cast<NM_Set_Time_Regulating*>(msg));
         break ;
 
       case NetworkMessage::SET_TIME_CONSTRAINED:
         D.Out(pdTrace, "SetTimeConstrained du federe %u.", msg->federate);
         auditServer.setLevel(8);
-        processSetTimeConstrained(msg);
+        
+        processSetTimeConstrained(link, static_cast<NM_Set_Time_Constrained*>(msg));
         break ;
 
       case NetworkMessage::PUBLISH_OBJECT_CLASS:
@@ -243,6 +245,20 @@ RTIG::chooseProcessingMethod(Socket *link, NetworkMessage *msg)
         processSubscribeInteractionClass(link, msg);
         break ;
 
+      case NetworkMessage::SET_CLASS_RELEVANCE_ADVISORY_SWITCH:
+	D[pdTrace] << "setClassRelevanceAdvisorySwitch (DM) " << 
+		static_cast<NM_Set_Class_Relevance_Advisory_Switch*>(msg)->
+		getClassRelevanceAdvisorySwitch() <<  endl;
+        auditServer.setLevel(6);
+	/*
+	 * Successful passing of services 
+	 * enableClassRelevanceAdvisorySwitch
+	 * and 
+	 * disableClassRelevanceAdvisorySwitch 
+	 * to RTIG
+	 */
+	break;
+	
       case NetworkMessage::REGISTER_OBJECT:
         D.Out(pdTrace, "registerObject.");
         auditServer.setLevel(6);
@@ -368,10 +384,10 @@ RTIG::chooseProcessingMethod(Socket *link, NetworkMessage *msg)
         auditServer.setLevel(6);
         processUnsubscribeInteractionWR(link, msg);
         break ;	
-	
+
       default:
         // FIXME: Should treat other cases CHANGE_*_ORDER/TRANSPORT_TYPE
-        D.Out(pdError, "processMessageRecu: unknown type %u.", msg->type);
+        D.Out(pdError, "processMessageRecu: unknown type %u.", msg->getType());
         throw RTIinternalError("Unknown Message Type");
     }
     G.Out(pdGendoc,"exit  RTIG::chooseProcessingMethod");
@@ -428,19 +444,37 @@ terminate = false ;
 
 while (!terminate) {
 	// Initialize fd_set structure with all opened sockets.
-	FD_ZERO(&fd);
-	FD_SET(tcpSocketServer.returnSocket(), &fd);
+	#if _WIN32
 
-	#if WIN32
-		 int highest_fd = socketServer.addToFDSet(&fd);
-		 int server_socket = tcpSocketServer.returnSocket();   
-		 highest_fd = server_socket>highest_fd ? server_socket : highest_fd;
-		 
-		 result = 0;	// Wait for an incoming message.
-		 result = select(highest_fd+1, &fd, NULL, NULL, NULL);
-		 
-		 if((result == -1)&&(WSAGetLastError() == WSAEINTR)) break;
+	result = 0;	// Wait for an incoming message.
+	while (!result)
+		{
+		int test;
+		
+		FD_ZERO(&fd);
+		FD_SET(tcpSocketServer.returnSocket(), &fd);
+
+		int highest_fd = socketServer.addToFDSet(&fd);
+		int server_socket = tcpSocketServer.returnSocket();
+
+		//typedef struct timeval {  long tv_sec;  long tv_usec; } 	 
+		timeval		watchDog;
+		watchDog.tv_sec= 0;
+		watchDog.tv_usec= 50000L;
+
+		highest_fd = server_socket>highest_fd ? server_socket : highest_fd;
+
+		result = select(highest_fd+1, &fd, NULL, NULL, &watchDog);
+		if (result < 0) test= WSAGetLastError();
+		if (terminate) break;
+		}
+	if (terminate) break;
+
+	if((result == -1)&&(WSAGetLastError() == WSAEINTR)) break;
 	#else
+		FD_ZERO(&fd);
+		FD_SET(tcpSocketServer.returnSocket(), &fd);
+
 		int fd_max = socketServer.addToFDSet(&fd);
 		fd_max = std::max(tcpSocketServer.returnSocket(), fd_max);
 	
@@ -508,11 +542,10 @@ caught by this module. Then a message, similar to the received one is sent
 on the link. This message only holds the exception.
 */
 Socket*
-RTIG::processIncomingMessage(Socket *link)
+RTIG::processIncomingMessage(Socket *link) throw (NetworkError)
 {
-    NetworkMessage msg ;
-    NetworkMessage rep ; // Server Answer(only if an exception is raised)
-
+    NetworkMessage* msg ;
+    
     char buffer[BUFFER_EXCEPTION_REASON_SIZE] ; // To store the exception reason
     G.Out(pdGendoc,"enter RTIG::processIncomingMessage");
     if (link == NULL) {
@@ -520,13 +553,14 @@ RTIG::processIncomingMessage(Socket *link)
         return NULL ;
     }
 
-    msg.read(link);
+    /* virtual constructor call */
+    msg = NM_Factory::receive(link);
+    
+    // Server Answer(only if an exception is raised)
+    std::auto_ptr<NetworkMessage> rep(NM_Factory::create(msg->getType()));    
+    rep->federate = msg->federate ;
 
-    rep.type = msg.type ;
-    rep.exception = e_NO_EXCEPTION ;
-    rep.federate = msg.federate ;
-
-    auditServer.startLine(msg.federation, msg.federate, msg.type);
+    auditServer.startLine(msg->federation, msg->federate, msg->getType());
 
     // This macro is used to copy any non null exception reason
     // string into our buffer(used for Audit purpose).
@@ -536,332 +570,332 @@ RTIG::processIncomingMessage(Socket *link)
     buffer[0] = 0 ;
 
     try {
-        link = chooseProcessingMethod(link, &msg);
+        link = chooseProcessingMethod(link, msg);
     }
     catch (ArrayIndexOutOfBounds &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ArrayIndexOutOfBounds ;
+        rep->exception = e_ArrayIndexOutOfBounds ;
     }
     catch (AttributeAlreadyOwned &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeAlreadyOwned ;
+        rep->exception = e_AttributeAlreadyOwned ;
     }
     catch (AttributeAlreadyBeingAcquired &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeAlreadyBeingAcquired ;
+        rep->exception = e_AttributeAlreadyBeingAcquired ;
     }
     catch (AttributeAlreadyBeingDivested &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeAlreadyBeingDivested ;
+        rep->exception = e_AttributeAlreadyBeingDivested ;
     }
     catch (AttributeDivestitureWasNotRequested &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeDivestitureWasNotRequested ;
+        rep->exception = e_AttributeDivestitureWasNotRequested ;
     }
     catch (AttributeAcquisitionWasNotRequested &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeAcquisitionWasNotRequested ;
+        rep->exception = e_AttributeAcquisitionWasNotRequested ;
     }
     catch (AttributeNotDefined &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeNotDefined ;
+        rep->exception = e_AttributeNotDefined ;
     }
     catch (AttributeNotKnown &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeNotKnown ;
+        rep->exception = e_AttributeNotKnown ;
     }
     catch (AttributeNotOwned &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeNotOwned ;
+        rep->exception = e_AttributeNotOwned ;
     }
     catch (AttributeNotPublished &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeNotPublished ;
+        rep->exception = e_AttributeNotPublished ;
     }
     catch (AttributeNotSubscribed &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_AttributeNotSubscribed ;
+        rep->exception = e_AttributeNotSubscribed ;
     }
     catch (ConcurrentAccessAttempted &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ConcurrentAccessAttempted ;
+        rep->exception = e_ConcurrentAccessAttempted ;
     }
     catch (CouldNotDiscover &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_CouldNotDiscover ;
+        rep->exception = e_CouldNotDiscover ;
     }
     catch (CouldNotOpenRID &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_CouldNotOpenRID ;
+        rep->exception = e_CouldNotOpenRID ;
     }
     catch (CouldNotOpenFED &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_CouldNotOpenFED ;
+        rep->exception = e_CouldNotOpenFED ;
     }
     catch (CouldNotRestore &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_CouldNotRestore ;
+        rep->exception = e_CouldNotRestore ;
     }
     catch (DeletePrivilegeNotHeld &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_DeletePrivilegeNotHeld ;
+        rep->exception = e_DeletePrivilegeNotHeld ;
     }
     catch (ErrorReadingRID &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ErrorReadingRID ;
+        rep->exception = e_ErrorReadingRID ;
     }
     catch (EventNotKnown &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_EventNotKnown ;
+        rep->exception = e_EventNotKnown ;
     }
     catch (FederateAlreadyPaused &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateAlreadyPaused ;
+        rep->exception = e_FederateAlreadyPaused ;
     }
     catch (FederateAlreadyExecutionMember &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateAlreadyExecutionMember ;
+        rep->exception = e_FederateAlreadyExecutionMember ;
     }
     catch (FederateDoesNotExist &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateDoesNotExist ;
+        rep->exception = e_FederateDoesNotExist ;
     }
     catch (FederateInternalError &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateInternalError ;
+        rep->exception = e_FederateInternalError ;
     }
     catch (FederateNameAlreadyInUse &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateNameAlreadyInUse ;
+        rep->exception = e_FederateNameAlreadyInUse ;
     }
     catch (FederateNotExecutionMember &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateNotExecutionMember ;
+        rep->exception = e_FederateNotExecutionMember ;
     }
     catch (FederateNotPaused &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateNotPaused ;
+        rep->exception = e_FederateNotPaused ;
     }
     catch (FederateNotPublishing &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateNotPublishing ;
+        rep->exception = e_FederateNotPublishing ;
     }
     catch (FederateNotSubscribing &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateNotSubscribing ;
+        rep->exception = e_FederateNotSubscribing ;
     }
     catch (FederateOwnsAttributes &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateOwnsAttributes ;
+        rep->exception = e_FederateOwnsAttributes ;
     }
     catch (FederatesCurrentlyJoined &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederatesCurrentlyJoined ;
+        rep->exception = e_FederatesCurrentlyJoined ;
     }
     catch (FederateWasNotAskedToReleaseAttribute &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederateWasNotAskedToReleaseAttribute ;
+        rep->exception = e_FederateWasNotAskedToReleaseAttribute ;
     }
     catch (FederationAlreadyPaused &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederationAlreadyPaused ;
+        rep->exception = e_FederationAlreadyPaused ;
     }
     catch (FederationExecutionAlreadyExists &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederationExecutionAlreadyExists ;
+        rep->exception = e_FederationExecutionAlreadyExists ;
     }
     catch (FederationExecutionDoesNotExist &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederationExecutionDoesNotExist ;
+        rep->exception = e_FederationExecutionDoesNotExist ;
     }
     catch (FederationNotPaused &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederationNotPaused ;
+        rep->exception = e_FederationNotPaused ;
     }
     catch (FederationTimeAlreadyPassed &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_FederationTimeAlreadyPassed ;
+        rep->exception = e_FederationTimeAlreadyPassed ;
     }
     catch (IDsupplyExhausted &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_IDsupplyExhausted ;
+        rep->exception = e_IDsupplyExhausted ;
     }
     catch (InteractionClassNotDefined &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InteractionClassNotDefined ;
+        rep->exception = e_InteractionClassNotDefined ;
     }
     catch (InteractionClassNotKnown &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InteractionClassNotKnown ;
+        rep->exception = e_InteractionClassNotKnown ;
     }
     catch (InteractionClassNotPublished &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InteractionClassNotPublished ;
+        rep->exception = e_InteractionClassNotPublished ;
     }
     catch (InteractionParameterNotDefined &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InteractionParameterNotDefined ;
+        rep->exception = e_InteractionParameterNotDefined ;
     }
     catch (InteractionParameterNotKnown &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InteractionParameterNotKnown ;
+        rep->exception = e_InteractionParameterNotKnown ;
     }
     catch (InvalidDivestitureCondition &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidDivestitureCondition ;
+        rep->exception = e_InvalidDivestitureCondition ;
     }
     catch (InvalidExtents &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidExtents ;
+        rep->exception = e_InvalidExtents ;
     }
     catch (InvalidFederationTime &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidFederationTime ;
+        rep->exception = e_InvalidFederationTime ;
     }
     catch (InvalidFederationTimeDelta &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidFederationTimeDelta ;
+        rep->exception = e_InvalidFederationTimeDelta ;
     }
     catch (InvalidObjectHandle &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidObjectHandle ;
+        rep->exception = e_InvalidObjectHandle ;
     }
     catch (InvalidOrderType &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidOrderType ;
+        rep->exception = e_InvalidOrderType ;
     }
     catch (InvalidResignAction &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidResignAction ;
+        rep->exception = e_InvalidResignAction ;
     }
     catch (InvalidRetractionHandle &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidRetractionHandle ;
+        rep->exception = e_InvalidRetractionHandle ;
     }
     catch (InvalidRoutingSpace &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidRoutingSpace ;
+        rep->exception = e_InvalidRoutingSpace ;
     }
     catch (InvalidTransportType &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_InvalidTransportType ;
+        rep->exception = e_InvalidTransportType ;
     }
     catch (MemoryExhausted &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_MemoryExhausted ;
+        rep->exception = e_MemoryExhausted ;
     }
     catch (NameNotFound &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_NameNotFound ;
+        rep->exception = e_NameNotFound ;
     }
     catch (NoPauseRequested &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_NoPauseRequested ;
+        rep->exception = e_NoPauseRequested ;
     }
     catch (NoResumeRequested &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_NoResumeRequested ;
+        rep->exception = e_NoResumeRequested ;
     }
     catch (ObjectClassNotDefined &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ObjectClassNotDefined ;
+        rep->exception = e_ObjectClassNotDefined ;
     }
     catch (ObjectClassNotKnown &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ObjectClassNotKnown ;
+        rep->exception = e_ObjectClassNotKnown ;
     }
     catch (ObjectClassNotPublished &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ObjectClassNotPublished ;
+        rep->exception = e_ObjectClassNotPublished ;
     }
     catch (ObjectClassNotSubscribed &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ObjectClassNotSubscribed ;
+        rep->exception = e_ObjectClassNotSubscribed ;
     }
     catch (ObjectNotKnown &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ObjectNotKnown ;
+        rep->exception = e_ObjectNotKnown ;
     }
     catch (ObjectAlreadyRegistered &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ObjectAlreadyRegistered ;
+        rep->exception = e_ObjectAlreadyRegistered ;
     }
     catch (RegionNotKnown &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_RegionNotKnown ;
+        rep->exception = e_RegionNotKnown ;
     }
     catch (RestoreInProgress &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_RestoreInProgress ;
+        rep->exception = e_RestoreInProgress ;
     }
     catch (RestoreNotRequested &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_RestoreNotRequested ;
+        rep->exception = e_RestoreNotRequested ;
     }
     catch (RTIinternalError &e) {
         if (e._reason == NULL)
@@ -869,108 +903,108 @@ RTIG::processIncomingMessage(Socket *link)
         else
             D.Out(pdExcept, "Catching \"%s\" exception: %s.", e._name, e._reason);
         CPY_NOT_NULL(e);
-        rep.exception = e_RTIinternalError ;
+        rep->exception = e_RTIinternalError ;
     }
     catch (SaveInProgress &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_SaveInProgress ;
+        rep->exception = e_SaveInProgress ;
     }
     catch (SaveNotInitiated &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_SaveNotInitiated ;
+        rep->exception = e_SaveNotInitiated ;
     }
     catch (SecurityError &e) {
         cout << endl << "Security Error : " << e._reason << endl ;
         CPY_NOT_NULL(e);
-        rep.exception = e_SecurityError ;
+        rep->exception = e_SecurityError ;
     }
     catch (SpaceNotDefined &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_SpaceNotDefined ;
+        rep->exception = e_SpaceNotDefined ;
     }
     catch (SpecifiedSaveLabelDoesNotExist &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_SpecifiedSaveLabelDoesNotExist ;
+        rep->exception = e_SpecifiedSaveLabelDoesNotExist ;
     }
     catch (TimeAdvanceAlreadyInProgress &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_TimeAdvanceAlreadyInProgress ;
+        rep->exception = e_TimeAdvanceAlreadyInProgress ;
     }
     catch (TimeAdvanceWasNotInProgress &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_TimeAdvanceWasNotInProgress ;
+        rep->exception = e_TimeAdvanceWasNotInProgress ;
     }
     catch (TooManyIDsRequested &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_TooManyIDsRequested ;
+        rep->exception = e_TooManyIDsRequested ;
     }
     catch (UnableToPerformSave &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_UnableToPerformSave ;
+        rep->exception = e_UnableToPerformSave ;
     }
     catch (UnimplementedService &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_UnimplementedService ;
+        rep->exception = e_UnimplementedService ;
     }
     catch (UnknownLabel &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_UnknownLabel ;
+        rep->exception = e_UnknownLabel ;
     }
     catch (ValueCountExceeded &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ValueCountExceeded ;
+        rep->exception = e_ValueCountExceeded ;
     }
     catch (ValueLengthExceeded &e) {
         D.Out(pdExcept, "Catching \"%s\" exception.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_ValueLengthExceeded ;
+        rep->exception = e_ValueLengthExceeded ;
     }
 
     // Non RTI specific exception, Client connection problem(internal)
     catch (NetworkError &e) {
         strcpy(buffer, " - NetworkError");
-        auditServer.endLine(rep.exception, buffer);
-
+        auditServer.endLine(rep->exception, buffer);
+        delete msg;
         throw e ;
     }
-
     // Default Handler
     catch (Exception &e) {
         D.Out(pdExcept, "Unknown Exception : %s.", e._name);
         CPY_NOT_NULL(e);
-        rep.exception = e_RTIinternalError ;
+        rep->exception = e_RTIinternalError ;
     }
 
     // buffer may contain an exception reason. If not, set it to OK
     // or Exception
     if (strlen(buffer)== 0) {
-        if (rep.exception == e_NO_EXCEPTION)
+        if (rep->exception == e_NO_EXCEPTION)
             strcpy(buffer, " - OK");
         else
             strcpy(buffer, " - Exception");
     }
 
-    auditServer.endLine(rep.exception, buffer);
-
+    auditServer.endLine(rep->exception, buffer);
+    delete msg;
     if (link == NULL) return link ;
 
-    if (rep.exception != e_NO_EXCEPTION) {
+    /* FIXME ***/
+    if (rep->exception != e_NO_EXCEPTION) {
         G.Out(pdGendoc,"            processIncomingMessage ===> write on exception to RTIA");
-        rep.write(link);
+        rep->send(link,NM_msgBufSend);
         D.Out(pdExcept,
               "RTIG catched exception %d and sent it back to federate %d.",
-              rep.exception, rep.federate);
+              rep->exception, rep->federate);
     }
     G.Out(pdGendoc,"exit  RTIG::processIncomingMessage");
     return link ;
@@ -978,17 +1012,17 @@ RTIG::processIncomingMessage(Socket *link)
 
 // ----------------------------------------------------------------------------
 //! process received signals.
-void
+ void
 RTIG::signalHandler(int sig)
 {
 D.Out(pdError, "Received Signal %d.", sig);
 
 if (sig == SIGINT) terminate = true ;
-#ifndef WIN32
+#ifndef _WIN32
 	if (sig == SIGPIPE) cout << "Ignoring 'Broken pipe' signal." << endl ;
 #endif
 }
 
 }} // namespace certi/rtig
 
-// $Id: RTIG.cc,v 3.39 2008/04/08 14:18:17 rousse Exp $
+// $Id: RTIG.cc,v 3.49 2008/07/10 20:20:05 approx Exp $
